#ifndef _AFFINE_H
#define _AFFINE_H

#include "matrix.h"
#include "list.h"

typedef struct _affine {
  matrix *cartan;		/* cartan matrix */
  matrix *tcartan;		/* transposed cartan matrix */
  int dcartan;			/* determinant of cartan matrix */
  matrix *acartan;		/* adjoint of cartan matrix */
  matrix *atcartan;		/* adjoint of transposed cartan matrix */
  char root_system_type;	/* one of 'A', 'B', 'C', 'D', 'E', 'F' */
  int *simple_length;		/* squares of lengths of the simple roots */
  matrix *root_dot;		/* root_dot[i,j] = 2 simp_i . simp_j */
  int wdot_factor;		/* dot products of fundamental weights: */
  matrix *weight_dot;		/* weight_dot[i,j] = wdot_factor w_i . w_j */
  
  int h;			/* coxeter number */
  int p;			/* prime number >= coxeter number */
  list *top_alcove;		/* affine weyl group elt for top alcove */
  list *longest_element;	/* longest element of Weyl group */
  list *positive_roots;		/* list of all positive roots */
  list *weight_roots;		/* positive roots in weight coordinates */
  list *root_length;		/* squared lengths of positive roots */
  list *permutations;		/* permutations of the dynkin diagram */
  
  vector *highest_root;		/* highest root in rootsystem */
  matrix *highest_elem;		/* highest_elem is a member of the */
  int highest_simp;		/* Weyl group that permutes the simple root 
				   number highest_simp to highest_root */
  
  vector *high_pair;		/* if x is in weight coordinates, 
				   <x, highest_root> = x . high_pair */
  vector *high_weight;		/* highest root in weight coordinates */
  vector *rho;			/* just the vector (1,1,...,1) */
  
  vector **cartan_rows;		/* list of the rows in the cartan matrix */
  vector **cartan_columns;	/* list of columns in the cartan matrix */
} affine;


matrix *mk_cartan(char *s);

#define afn_dim(afn)	(m_dim((afn)->cartan))

affine *init_affine(char *s);
void afn_free(affine *afn);

#endif
