/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "alloc.h"
#include "list.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "calc.h"
#include "tree.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s [-drwc] rsys-p wge\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  list *s;
  affine *afn;
  treebase *tb;
  int filter = TB_ALL;
  int mode = PTM_WEYL;
  int c;
  
  cmd_init(ac, av);
  while ((c = getopt(ac, av, "drwc")) != EOF)
    switch (c)
      {
      case 'd':
	filter = TB_DOMINANT;
	break;
	
      case 'r':
	filter = TB_RESTRICTED;
	break;
	
      case 'w':
	mode = PTM_WEIGHT;
	break;
	
      case 'c':
	mode = PTM_COORD;
	break;
	
      default:
	print_usage(av[0]);
      }
  
  afn = cmd_rsys();
  s = cmd_wge(afn);
  
  tb = build_tree(s, filter, afn);
  print_tree(tb, mode);
  
  tb_free(tb);
  l_free(s);
  memory_report;
  
  exit(0);
}
