#ifndef _FILES_H
#define _FILES_H

#include <stdio.h>

#include "list.h"
#include "matrix.h"

extern char fn_digit[];

enum {
  SET_DEFAULT_BITS,
  EXCPT_8BIT_ARG,
  EXCPT_16BIT_ARG,
  EXCPT_24BIT_ARG,
  EXCPT_32BIT_ARG,
  FLUSH_FILE,
  END_OF_FILE
};

#define BF_EOF		1
#define BF_READ		0
#define BF_WRITE	2
#define BF_DONT_CLOSE	4
#define BF_FLUSH	8

typedef struct {
  FILE *stream;
  int flags;
  int dflt_bits;
  int bits_used;
  unsigned buf;
} binfile;

#define bf_stream(bf)		((bf)->stream)
#define bf_flags(bf)		((bf)->flags)
#define bf_dflt_bits(bf)	((bf)->dflt_bits)
#define bf_bits_used(bf)	((bf)->bits_used)
#define bf_buf(bf)		((bf)->buf)
#define bf_eof(bf)		((bf)->flags & BF_EOF)
#define bf_isflushed(bf)	((bf)->flags & BF_FLUSH)
#define bf_clearflush(bf)	((bf)->flags &= ~BF_FLUSH)

int unsigned_bits(unsigned maxval);

binfile *bf_fopenwrite(FILE *stream, int dflt_bits, int flags);
binfile *bf_openwrite(char *fname, int dflt_bits);
binfile *bf_fopenread(FILE *stream, int flags);
binfile *bf_openread(char *fname);
void bf_flush(binfile *bf);
void bf_close(binfile *bf);

void put_dflt_bits(binfile *bf, int dflt_bits);

void put_bits(binfile *bf, unsigned x, int nbits);
unsigned get_bits(binfile *bf, int nbits);

void put_int(binfile *bf, int i);
void put_uint(binfile *bf, unsigned i);
void put_vec(binfile *bf, vector *v);
void put_uvec(binfile *bf, vector *v);
void put_mat(binfile *bf, matrix *m);
void put_intlist(binfile *bf, list *l);
void put_ulist(binfile *bf, list *l);
void put_string(binfile *bf, char *s);

int get_int(binfile *bf);
unsigned get_uint(binfile *bf);
void get_vec(binfile *bf, vector *v);
void get_uvec(binfile *bf, vector *v);
void get_mat(binfile *bg, matrix *m);
void get_intlist(binfile *bf, list *l);
void get_ulist(binfile *bf, list *l);
void get_string(binfile *bf, char *s);

#endif
