/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>

#include "alloc.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "weight.h"
#include "calc.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys-p\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  int n;
  affine *afn;
  list *exp;
  vector *a, *lambda;
  weight_set *ws;
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  n = m_dim(afn->cartan);
  
  exp = l_new();
  lambda = v_new(n);
  a = v_new(n);
  v_copy(a, afn->rho);
  v_scale(a, afn->p, a);
  
  v_set_zero(lambda);
  ws = ws_new(lambda);
  
  while (v_compare(a, afn->rho) != 0)
    {
      int i;
      
      weight_to_exp(exp, a, NULL, afn);
      calc_alcove(lambda, exp, afn);
      v_sub(lambda, lambda, afn->rho);
      ws_insert(ws, lambda);
      
      i = 0;
      while (v_entry(a, i) == 1)
	v_entry(a, i++) = afn->p;
      v_entry(a, i)--;
    }
  
  print_weight_set(ws, 0, 0);
  
  afn_free(afn);
  l_free(exp);
  v_free(lambda);
  v_free(a);
  ws_free(ws);
  
  memory_report;
  
  exit(0);
}
