/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include "janzlib.h"
#include "list.h"
#include "calc.h"
#include "alloc.h"

janzdata *jd_new(weight_set *sum, list *js_list, list *nd_list)
{
  janzdata *jd = (janzdata *) weyl_calloc(1, sizeof(janzdata));
  jd_jsum(jd) = sum;
  jd_jslist(jd) = js_list;
  jd_ndlist(jd) = nd_list;
  return jd;
}

void jd_free(janzdata *jd)
{
  ws_free(jd_jsum(jd));
  weyl_free(jd);
}

void init_janzdata(treebase *tb)
{
  int l;
  node *nd, *nd2;
  for (l = 0; l <= tb_max_length(tb); l++)
    for (nd = tb_node_list(tb, l); nd != NULL; nd = n_next(nd))
      {
	weight_set *sum = jantzen_sum(n_alcove(nd), tb_affine(tb));
	list *js_list = ws_to_list(sum);
	list *nd_list = l_new();
	int i;
	for (i = 0; i < l_length(js_list); i++)
	  {
	    weight_set *ws = l_element(js_list, i);
	    vector *w = ws_weight(ws);
	    nd2 = tb_findalc(tb, w);
	    l_appendto(nd_list, nd2);
	  }
	n_data(nd) = jd_new(sum, js_list, nd_list);
      }
}

static int ny_p(int m, int p)
{
  int r = 0;
  while (m % p == 0)
    {
      m /= p;
      r++;
    }
  return r;
}

static int neg_index(vector *w)
{
  int i;
  for (i = l_length(w); i > 0 && v_entry(w, i - 1) >= 0; i--)
    ;
  return i;
}

static int zero_index(vector *w)
{
  int i;
  for (i = l_length(w); i > 0 && v_entry(w, i - 1) != 0; i--)
    ;
  return i;
}

weight_set *jantzen_sum(vector *lambda, affine *afn)
{
  int i, n = m_dim(afn->cartan);
  list *plusroots = afn->positive_roots;
  list *weight_roots = afn->weight_roots;
  weight_set *sum = NULL;
  
  vector *mu = v_new(n);
  vector *w = v_new(n);
  
  for (i = 0; i < l_length(plusroots); i++)
    {
      vector *alpha = l_element(plusroots, i);
      vector *alpha_w = l_element(weight_roots, i);
      int pair = v_dot(lambda, alpha);
      int m;
      v_copy(mu, lambda);
      v_add_multiple(mu, - pair, alpha_w);
      
      for (m = 1; m * afn->p < pair; m++)
	{
	  int s, coef = 1 + ny_p(m, afn->p);
	  v_add_multiple(mu, afn->p, alpha_w);
	  v_copy(w, mu);
	  while ((s = neg_index(w)) != 0)
	    {
	      ref_weight(w, s, afn);
	      coef = -coef;
	    }
	  if (zero_index(w) == 0)
	    {
	      weight_set *ws;
	      
	      if (sum == NULL)
		{
		  sum = ws_new(w);
		  ws = sum;
		}
	      else
		{
		  ws = ws_insert(sum, w);
		  if (ws == NULL)
		    ws = ws_find(sum, w);
		}
	      ws_mult(ws) += coef;
	    }
	}
    }
  
  v_free(mu);
  v_free(w);
  
  return sum;
}
