/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <stdlib.h>

#include "alloc.h"
#include "list.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "calc.h"
#include "tree.h"
#include "kllib.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys wge1 - wge2\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  list *s1, *s2;
  treebase *tb;
  polynomial *P;
  affine *afn;
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  s1 = cmd_wge(afn);
  cmd_dash(1);
  s2 = cmd_wge(afn);
  
  tb = build_tree(s2, 0, afn);
  
  calc_bit_matrices(tb);
  tb_init_colbits(tb);
  
  P = kl_polynomial(s1, s2, tb);
  
  print_poly(P);
  
#if 0
  hash_print_stat(tb_nodeset(tb), 10);
  hash_print_stat(tb_kl_cache(tb), 10);
#endif
  
  l_free(s1);
  l_free(s2);
  tb_free(tb);
  
  memory_report;
  
  exit(0);
}
