/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "alloc.h"
#include "list.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "calc.h"
#include "tree.h"
#include "kllib.h"
#include "weight.h"
#include "freudlib.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s [-s] rsys\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  treebase *tb, *tabtree;
  list *w0, *top, *w0top, *y, *w0y, *x, *w0x;
  node *ny, *nx;
  polynomial *P;
  vector *a;
  int c, opt_short;
  int n, i, l;
  affine *afn;
  
  cmd_init(ac, av);
  
  c = getopt(ac, av, "s");
  if (c == '?')
    print_usage(av[0]);
  opt_short = (c == 's');
  
  afn = cmd_rsys();
  
  w0 = afn->longest_element;
  top = afn->top_alcove;
  n = afn_dim(afn);
  
  tabtree = build_tree(top, TB_DOMINANT, afn);
  
  w0top = l_new();
  l_copy(w0top, w0);
  for (i = 0; i < l_length(top); i++)
    l_appendto(w0top, l_element(top, i));
  reduced_exp(w0top, w0top, afn);

  tb = build_tree(w0top, TB_ALL, afn);
  calc_bit_matrices(tb);
  tb_init_colbits(tb);

  y = l_new();
  w0y = l_new();
  x = l_new();
  w0x = l_new();
  
  for (l = 0; l <= tb_max_length(tabtree); l++)
    for (ny = tb_node_list(tabtree, l); ny != NULL; ny = n_next(ny))
      {
	a = n_alcove(ny);
	for (i = v_length(a); i > 0 && v_entry(a, i-1) <= afn->p; i--)
	  ;
	if (i == 0)
	  {
	    int j, maxlen;
	    
	    alcove_to_exp(y, a, afn);
	    l_copy(w0y, w0);
	    for (j = 0; j < l_length(y); j++)
	      l_appendto(w0y, l_element(y, j));
	    reduced_exp(w0y, w0y, afn);
	    
	    maxlen = n_length(ny);
	    if (opt_short)
	      maxlen -= 3;
	    
	    for (j = 0; j <= maxlen; j++)
	      for (nx = tb_node_list(tabtree, j); nx != NULL; nx = n_next(nx))
		{		
		    alcove_to_exp(x, n_alcove(nx), afn);
		    
		    if (bruhat_lesseq(x, y, tb))
		      {
			int t;
			
			l_copy(w0x, w0);
			for (t = 0; t < l_length(x); t++)
			  l_appendto(w0x, l_element(x, t));
			reduced_exp(w0x, w0x, afn);
			
			P = kl_polynomial(w0x, w0y, tb);
			
			printf("P(long");
			for (t = 0; t < l_length(x); t++)
			  printf(" %d", l_intelem(x, t));
			printf(", long");
			for (t = 0; t < l_length(y); t++)
			  printf(" %d", l_intelem(y, t));
			printf(") = ");
			print_poly(P);
		      }
		  }
	  }
      }
  
  l_free(w0top);
  l_free(w0y);
  l_free(w0x);
  l_free(y);
  l_free(x);
  tb_free(tb);
  tb_affine(tabtree) = NULL;
  tb_free(tabtree);
  
  memory_report;
  
  exit(0);
}

