/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdlib.h>
#include <string.h>

#include "alloc.h"
#include "matrix.h"
#include "list.h"

#define _LIST_INITIAL_LENGTH	16

list *l_new()
{
  list *lst = (list *) weyl_malloc(sizeof(list));
  l_length(lst) = 0;
  lst->array_size = _LIST_INITIAL_LENGTH;
  lst->array = weyl_malloc(_LIST_INITIAL_LENGTH * sizeof(void *));
  return lst;
}

list *l_new_copy(list *src)
{
  list *dst = (list *) weyl_malloc(sizeof(list));
  dst->length = l_length(src);
  dst->array_size = l_length(src) + 1;
  dst->array = weyl_malloc(dst->array_size * sizeof(void *));
  memcpy(dst->array, src->array, l_length(src) * sizeof(void *));
  return dst;
}

void l_free(list *list)
{
  weyl_free(list->array);
  weyl_free(list);
}

void l_copy(list *dst, list *src)
{
  if (l_length(src) > dst->array_size)
    {
      dst->array_size = 2 * l_length(src);
      dst->array = 
	weyl_realloc(dst->array, dst->array_size * sizeof(void *));
    }
  l_length(dst) = l_length(src);
  memcpy(dst->array, src->array, l_length(src) * sizeof(void *));
}

void l_appendto(list *list, void *e)
{
  if (l_length(list) == list->array_size)
    {
      list->array_size *= 2;
      list->array = 
	weyl_realloc(list->array, list->array_size * sizeof(void *));
    }
  l_element(list, l_length(list)++) = e;
}

void l_remove_elem(list *list, int i)
{
  int len = --l_length(list);
  if (i != len)
    memmove(list->array + i, list->array + i + 1, (len - i) * sizeof(int));
}

void l_reverse(list *list)
{
  int n = l_length(list);
  int ilim = n / 2;
  int i;
  for (i = 0; i < ilim; i++)
    {
      void *tmp = l_element(list, i);
      l_element(list, i) = l_element(list, n - 1 - i);
      l_element(list, n - 1 - i) = tmp;
    }
}

int l_compare(list *lst1, list *lst2)
{
  int ln1 = l_length(lst1);
  int ln2 = l_length(lst2);
  int i;
  
  if (ln1 != ln2)
    return ln1 - ln2;
  
  for (i = 0; i < ln1; i++)
    {
      int d = l_intelem(lst1, i) - l_intelem(lst2, i);
      if (d) return d;
    }      
  
  return 0;
}


void free_veclist(list *list)
{
  int i;
  for (i = 0; i < l_length(list); i++)
    v_free(l_element(list, i));
  l_free(list);
}
