/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <unistd.h>

#include "list.h"
#include "alloc.h"
#include "matrix.h"
#include "file.h"
#include "output.h"
#include "rank.h"
#include "entrynum.h"
#include "cmdline.h"

void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s file.mat\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  binfile *bf;
  char s[255], *name;
  int i, j, n, p, dim, short_form;
  int first_entry, last_entry, entries_read, entries_total, percent;
  vector *v;
  list *buf;
  int c;
  
  short_form = 0;
  
  cmd_init(ac, av);
  while ((c = getopt(ac, av, "s")) != EOF)
    {
      if (c == 's')
	short_form = 1;
      else
	print_usage(av[0]);
    }
  name = cmd_file(1);
  
  if ((bf = bf_openread(name)) == NULL)
    cmd_error("Bad matrix file `%s'\n", name);
  
  get_string(bf, s);
  n = atoi(s + 1);
  
  p = get_uint(bf);
  if (! short_form)
    printf("Root system:  %s, p = %d\n", s, p);
  else
    printf("%s %d ", s, p);
  
  v = v_new(n);
  get_uvec(bf, v);
  if (! short_form)
    printf("lambda = ");
  for (i = 0; i < n; i++)
    putchar(fn_digit[v_entry(v, i)]);
  get_uvec(bf, v);
  if (! short_form)
    printf(" ;  mu = ");
  else
    putchar(':');
  for (i = 0; i < n; i++)
    putchar(fn_digit[v_entry(v, i)]);
  if (! short_form)
    putchar('\n');
  
  dim = get_uint(bf);
  if (! short_form)
    printf("Matrix dimension:  %d\n", dim);
  
  first_entry = get_uint(bf);
  number_to_entry(first_entry, &i, &j);
  if (! short_form)
    printf("First entry:       %d  (%d,%d)\n", first_entry, i, j);

  entries_read = 0;
  buf = l_new();
  while (! bf_eof(bf))
    {
      int x = get_uint(bf);
      if (bf_isflushed(bf))
	{
	  entries_read += l_length(buf);
	  l_reset(buf);
	  bf_clearflush(bf);
	}
      l_appendint(buf, x);
    }
  
  last_entry = first_entry + entries_read - 1;
  number_to_entry(last_entry, &i, &j);
  if (! short_form)
    {
      printf("Last entry:        %d  (%d,%d)\n", last_entry, i, j);
  
      entries_total = dim * (dim + 1) / 2;
      percent = 1000 * entries_read / entries_total;
      printf("Entries in file:   %d  of  %d   (%d.%d%%)\n", entries_read, 
	     entries_total, percent / 10, percent % 10);
    }
  else
    printf("  %9d -%9d\n", first_entry, last_entry);
  
  l_free(buf);
  v_free(v);
  bf_close(bf);
  
  memory_report;
  
  exit(0);
}
