/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>
#include <unistd.h>

#include "list.h"
#include "alloc.h"
#include "matrix.h"
#include "file.h"
#include "output.h"
#include "rank.h"
#include "entrynum.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s [-s] file.mat\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  binfile *bf;
  char s[255], *name;
  int i, j, n, p, dim, a, b;
  int entry_number;  
  vector *v;
  list *buf;
  int short_form = 0;
  int c;
  
  cmd_init(ac, av);
  while ((c = getopt(ac, av, "s")) != EOF)
    {
      if (c == 's')
	short_form = 1;
      else
	print_usage(av[0]);
    }
  name = cmd_file(1);
  
  if ((bf = bf_openread(name)) == NULL)
    cmd_error("Bad matrix file `%s'\n", name);
  
  get_string(bf, s);
  n = atoi(s + 1);
  
  p = get_uint(bf);
  printf("Root system:  %s, p = %d\n", s, p);
  
  v = v_new(n);
  get_uvec(bf, v);
  printf("lambda = ");
  for (i = 0; i < n; i++)
    putchar(fn_digit[v_entry(v, i)]);
  get_uvec(bf, v);
  printf(" ;  mu = ");
  for (i = 0; i < n; i++)
    putchar(fn_digit[v_entry(v, i)]);
  putchar('\n');
  
  dim = get_uint(bf);
  printf("Matrix dimension:  %d\n", dim);
  
  entry_number = get_uint(bf);
  
  if (short_form)
    number_to_entry(entry_number, &a, &b);
  
  buf = l_new();
  while (! bf_eof(bf))
    {
      int x = get_uint(bf);
      if (bf_isflushed(bf))
	{
	  for (j = 0; j < l_length(buf); j++)
	    if (short_form)
	      {
		printf("%d", l_intelem(buf, j));
		if (b++ == a)
		  {
		    a++;
		    b = 0;
		    putchar('\n');
		  }
	      }
	    else
	      printf("%10d  %d\n", entry_number++, l_intelem(buf, j));
	 
	  l_reset(buf);
	  bf_clearflush(bf);
	}
      l_appendint(buf, x);
    }
  
  l_free(buf);
  v_free(v);
  bf_close(bf);
  
  memory_report;
  
  exit(0);
}
