/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <string.h>

#include "alloc.h"
#include "polynomial.h"

#define DEFAULT_INITIAL_DEGREE	8

polynomial *p_new(int c)
{
  polynomial *p = p_new_size(DEFAULT_INITIAL_DEGREE);
  p_set_const(p, c);
  return p;
}

void p_free(polynomial *p)
{
  weyl_free(p_coef_list(p));
  weyl_free(p);
}

polynomial *p_new_size(int sz)
{
  polynomial *p = (polynomial *) weyl_malloc(sizeof(polynomial));
  p_coef_list(p) = (int *) weyl_malloc(sz * sizeof(int));
  p_coef_size(p) = sz;
  p_coef_used(p) = 0;
  return p;
}

void p_extend(polynomial *p, int sz)
{
  if (p_coef_size(p) >= sz)
    return;
  sz *= 2;
  p_coef_list(p) = (int *) weyl_realloc(p_coef_list(p), sz * sizeof(int));
  p_coef_size(p) = sz;
}

#define m_min(x,y)	((x) < (y) ? (x) : (y))
#define m_max(x,y)	((x) > (y) ? (x) : (y))

void p_set_const(polynomial *p, int c)
{
  if (c == 0)
    p_coef_used(p) = 0;
  else
    {
      p_coef_used(p) = 1;
      p->coef_list[0] = c;
    }
}

void p_copy(polynomial *dst, polynomial *p)
{
  int used = p_coef_used(p);
  p_extend(dst, used);
  p_coef_used(dst) = used;
  if (used > 0)
    memcpy(p_coef_list(dst), p_coef_list(p), used * sizeof(int));
}

void p_add_multiple(polynomial *dst, polynomial *p, int r, int d)
{
  int i, dst_used;
  int used1 = p_coef_used(dst);
  int used2 = p_coef_used(p);
  
  if (r == 0 || used2 == 0)
    return;
  
  dst_used = m_max(used1, used2 + d);
  p_extend(dst, dst_used);
  
  for (i = used1; i < d; i++)
    p_coef(dst, i) = 0;
  
  for (i = d; i < dst_used; i++)
    {
      int x = 0;
      if (i < used1)
	x += p_coef(dst, i);
      if (i - d < used2)
	x += r * p_coef(p, i - d);
      p_coef(dst, i) = x;
    }
  
  while (dst_used > 0 && p_coef(dst, dst_used - 1) == 0)
    dst_used--;
  
  p_coef_used(dst) = dst_used;
}
