/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdlib.h>

#include "rational.h"

#ifdef __GNUC__
inline
#endif
int i_gcd(int x, int y)
{
  while (x != 0)
    {
      int z = y % x;
      y = x;
      x = z;
    }
  return abs(y);
}

small_rat r_new(int p, int q)
{
  small_rat res;
  int gcd = i_gcd(p, q);
  if (q > 0)
    {
      res.p = p / gcd;
      res.q = q / gcd;
    }
  else
    {
      res.p = -p / gcd;
      res.q = -q / gcd;
    }
  return res;
}

#ifdef __GNUC__
inline
#endif
small_rat r_add(small_rat a, small_rat b)
{
  return r_new(a.p * b.q + b.p * a.q, a.q * b.q);
}

#ifdef __GNUC__
inline
#endif
small_rat r_sub(small_rat a, small_rat b)
{
  return r_new(a.p * b.q - b.p * a.q, a.q * b.q);
}

#ifdef __GNUC__
inline
#endif
small_rat r_mult(small_rat a, small_rat b)
{
  return r_new(a.p * b.p, a.q * b.q);
}

#ifdef __GNUC__
inline
#endif
small_rat r_div(small_rat a, small_rat b)
{
  return r_new(a.p * b.q, a.q * b.p);
}

#ifdef __GNUC__
inline
#endif
small_rat r_iadd(int x, small_rat a)
{
  a.p += x * a.q;
  return a;
}

#ifdef __GNUC__
inline
#endif
small_rat r_isub(int x, small_rat a)
{
  a.p -= x * a.q;
  return a;
}

#ifdef __GNUC__
inline
#endif
small_rat r_imult(int x, small_rat a)
{
  return r_new(x * a.p, a.q);
}

#ifdef __GNUC__
inline
#endif
small_rat r_idiv(small_rat a, int x)
{
  return r_new(a.p, x * a.q);
}


#ifdef __GNUC__
inline
#endif
int r_eq(small_rat a, small_rat b)
{
  return a.p == b.p && a.q == b.q;
}

#ifdef __GNUC__
inline
#endif
int r_lt(small_rat a, small_rat b)
{
  return a.p * b.q < b.p * a.q;
}

#ifdef __GNUC__
inline
#endif
int r_leq(small_rat a, small_rat b)
{
  return a.p * b.q <= b.p * a.q;
}
