/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>

#include "alloc.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "rank.h"
#include "redulib.h"
#include "cmdline.h"


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys-p weight\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  affine *afn;
  int n, i, j, l;
  vector *a, *aw, *lambda;
  matrix *M;
  list *sums;
  int nsums;
  vector *inv;
  weight_set *hw_set;
  list *omega;
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  lambda = cmd_weight(0, afn);
  n = m_dim(afn->cartan);
  
  init_redulib(lambda, afn);
  
  inv = calc_inverse_list(afn->p);
  
  hw_set = calc_weights_below(lambda, afn);
  omega = ws_to_list(hw_set);
  
  a = v_new(n);
  aw = v_new(n);
  
  for (l = 0; l < l_length(omega); l++)
    {
      weight_set *ws = l_element(omega, l);
      vector *w = ws_weight(ws);
      
      v_sub(aw, lambda, w);
      m_multvec(a, afn->atcartan, aw);
      v_div(a, a, afn->dcartan);
      
      sums = generate_sums(a, afn);
      nsums = l_length(sums);
      M = m_new(nsums);
      
      for (i = 0; i < nsums; i++)
	for (j = 0; j <= i; j++)
	  {
	    vector *sum1 = l_element(sums, i);
	    vector *sum2 = l_element(sums, j);
	    m_entry(M, i, j) = m_entry(M, j, i) = reduce(sum1, sum2);
	    free_efl_trees();
	  }
      
      /* print_mat(M); */
      reduce_mat_mod_p(M, afn->p);
      /* printf("\n"); */
      /* print_mat(M); */
      
      print_weight(w, 0);
      printf(" :  %d\n", rank_mod_p(M, afn->p, inv));
      
      m_free(M);
      free_sums(sums);
    }
  
  afn_free(afn);
  v_free(lambda);
  v_free(a);
  v_free(aw);
  ws_free(hw_set);
  free_redulib();
  l_free(omega);
  v_free(inv);
  
  memory_report;
  
  exit(0);
}
