/*  Dynkin project programs
 *  Copyright (C) 1998 Anders S. Buch (abuch@mi.aau.dk)
 *  See the file LICENSE for license information.
 */

#include <stdio.h>

#include "alloc.h"
#include "matrix.h"
#include "affine.h"
#include "output.h"
#include "rank.h"
#include "freudlib.h"
#include "redulib2.h"
#include "entrynum.h"
#include "signals.h"
#include "file.h"
#include "cmdline.h"


#define FLUSH_PERIOD	500


void print_usage(char *s)
{
  fprintf(stderr, "Usage: %s rsys-p lambda mu [first [last]]\n", s);
  exit(1);
}

int main(int ac, char **av)
{
  affine *afn;
  int n, i, j, cnt, nsums;
  vector *a, *aw, *lambda, *mu;
  list *sums;
  binfile *bf;
  char filename[255];
  int entry, first_entry, last_entry, entries_total;
  
  cmd_init(ac, av);
  afn = cmd_rsys();
  n = m_dim(afn->cartan);
  lambda = cmd_weight(0, afn);
  cmd_dash(0);
  mu = cmd_weight(0, afn);
  cmd_dash(0);
  
  first_entry = (optind < ac) ? atoi(av[optind++]) : 0;
  last_entry  = (optind < ac) ? atoi(av[optind++]) : -1;
  
  a = v_new(n);
  aw = v_new(n);
  v_sub(aw, lambda, mu);
  m_multvec(a, afn->atcartan, aw);
  v_div(a, a, afn->dcartan);
  v_free(aw);
  
  sums = generate_sums(a, afn);
  nsums = l_length(sums);
  entries_total = nsums * (nsums + 1) / 2;
  if (last_entry == -1 || last_entry >= entries_total)
    last_entry = entries_total - 1;
  v_free(a);
  
  j = sprintf(filename, "%c%d-%d-", 
	      afn->root_system_type, m_dim(afn->cartan), afn->p);
  for (i = 0; i < n; i++)
    filename[j++] = fn_digit[v_entry(lambda, i)];
  filename[j++] = '-';
  for (i = 0; i < n; i++)
    filename[j++] = fn_digit[v_entry(mu, i)];
  sprintf(filename + j, "-%d-%d.mat", first_entry, last_entry);
  
  bf = bf_openwrite(filename, 8);
  sprintf(filename, "%c%d", afn->root_system_type, m_dim(afn->cartan));
  put_string(bf, filename);
  put_uint(bf, afn->p);
  put_uvec(bf, lambda);
  put_uvec(bf, mu);
  
  put_uint(bf, nsums);
  put_uint(bf, first_entry);
  
  i = unsigned_bits(afn->p);
  put_dflt_bits(bf, i);
  
  init_redulib2(lambda, afn);
  v_free(lambda);
  v_free(mu);
  afn_free(afn);
  
  init_signal_handler();
  
  cnt = 0;
  entry = first_entry;
  number_to_entry(entry, &i, &j);
  
  while (entry <= last_entry && ! term_signal)
    {
      vector *sum1 = l_element(sums, i);
      vector *sum2 = l_element(sums, j);
      int x = reduce2(sum1, sum2);
      put_uint(bf, x);
      
      entry++;
      if (++j > i)
	{
	  j = 0;
	  i++;
	}
      
      if (++cnt == FLUSH_PERIOD || hup_signal)
	{
	  bf_flush(bf);
	  cnt = 0;
	  hup_signal = 0;
	}
    }
  
  bf_close(bf);
  free_sums(sums);
  free_redulib2();
  
  memory_report;
  
  exit(0);
}
